/*
 *+------------------------------------------------------------------------+
 *| Licensed Materials - Property of IBM                                   |
 *|                                                                        |
 *| com.ibm.team.vvc.shared.web                                            |
 *|                                                                        |
 *| Copyright IBM Corporation 2012.  All Rights Reserved.                  |
 *|                                                                        |
 *| US Government Users Restricted Rights - Use, duplication or disclosure |
 *| restricted by GSA ADP Schedule Contract with IBM Corp.                 |
 *+------------------------------------------------------------------------+
 */
(function() {	

	dojo.provide("com.ibm.team.vvc.shared.web.ui.VVC_Utils");
	
	dojo.require("dojo.date.locale");
	
	var contextRoot = net.jazz.ajax._contextRoot;
	if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs';
		
	com.ibm.team.vvc.shared.web.ui.VVC_Utils = {
		VVC_NAMESPACE : "http://jazz.net/ns/vvc#", 
		RDF_NAMESPACE : "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
		DCTERMS_NAMESPACE : "http://purl.org/dc/terms/", 
		FOAF_NAMESPACE : "http://xmlns.com/foaf/0.1/",
		
		formatDate: function(dateToFormat)
		{
			return dojo.date.locale.format( new Date(dateToFormat), { formatLength: 'medium' } //$NON-NLS-1$
			);   
		},
		
		getDctermsDescriptionOnVersionedResource: function(resourceUri, conceptUri)
		{
			return this._getResourceLabel(resourceUri, "title", this.DCTERMS_NAMESPACE, conceptUri);
		},
		
		getDctermsDescription: function(resourceUri)
		{
			return this._getResourceLabel(resourceUri, "title", this.DCTERMS_NAMESPACE);
		},
		
		getFoafName: function(resourceUri)
		{
			return this._getResourceLabel(resourceUri, "name", this.FOAF_NAMESPACE);
		},
		
		_getResourceLabel: function(resourceUri, tagName, tagNamespace, conceptUri)
		{
			this.label = resourceUri;
			if (conceptUri != null)
				this.resource = conceptUri;
			else
				this.resource = resourceUri;
			
			this.resource = decodeURIComponent(this.resource);
			
			try {
				jazz.client.xhrGet({
					headers: {
						"vvc_servicePath" : resourceUri,
						"Accept": " application/rdf+xml"
					},
					handleAs: 'text',
					sync: true,
					failOk: true,
					url: contextRoot + '/vvcshared/proxy'+
						//TODO: Temporary work around to get around firebug ajax caching problem, because firebug is so dumb as to not respect the response headers
						'?right_now='+new Date().getTime(), //$NON-NLS-1$

					load: dojo.hitch(this, function(result) {
						var doc = dojox.xml.DomParser.parse(result);
						
						var entries = doc.getElementsByTagNameNS("RDF", this.RDF_NAMESPACE)[0].getElementsByTagNameNS("Description", this.RDF_NAMESPACE);
						if (entries.length>0){
							for (var i = 0; i<entries.length; i++) {
								var rdfAbout = entries[i].getAttribute("rdf:about");
								if (rdfAbout == this.resource)
								{
									var node = entries[i].getElementsByTagNameNS(tagName, tagNamespace);
									if (node.length > 0){
										var titleNode = node[0].childNodes[0];
										if (titleNode != null)
											this.label = titleNode.nodeValue;
										break;
									}
								}
							}
						}
					}),
					error: dojo.hitch(this, this._onError),
				});
			} catch (err){
			}
			
			return this.label; 
		},
		
		_onError: function(response, ioArgs) {
			console.log("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'...Invalid Response Status("+ioArgs.xhr.status+"): "+ response.message);
		},
	};

	
}());