/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid");

dojo.require("dijit._Widget");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dojox.grid.LazyTreeGridStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require('dojo.i18n');
dojo.require("jazz.util.html");
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.ui.PageTemplate");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

(function ()
{
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	var States = {
		Active: "IN_PROGRESS",
		Completed: "COMPLETED",
		Committed: "COMMITTED",
		Suspended: "SUSPENDED"
	};
	
	var Actions = {
			Commit: "COMMIT",
			Complete: "COMPLETE",
			Rename: "RENAME"
	};

	var contextRoot = net.jazz.ajax._contextRoot;
	if (contextRoot.indexOf('jazz') != -1) contextRoot += '/jfs';

	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid", [dijit._Widget], {
		
		_showCommitted: false,
		
		constructor: function (args)
		{
			if (args)
			{
				if (args.displayErrorMessage)
					this.displayErrorMessage = args.displayErrorMessage;
				if (args.hideErrorMessage)
					this.hideErrorMessage = args.hideErrorMessage;
			}
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructGrid();
		},
	   		
		_constructGrid: function() {
			if (!this._grid)
			{
				var model = this._getModel();
		
			    /* set up layout */
			    var layout = [
			                  {name: messages.actions, field: null, width: '130px', formatter: dojo.hitch(this, this.formatActions)},
			                  {name: messages.description, field: 'title', width: 'auto', formatter: dojo.hitch(this, this.formatName)},
			                  {name: messages.ownedBy, field: 'creatorName', width: '200px', formatter: dojo.hitch(this, this.formatUserName)}
			    ];
		
			    this._grid = new dojox.grid.LazyTreeGrid({
			        id: 'changeSetGrid',
			        treeModel: model,
			        structure: layout,
			        rowSelector: '0px',
			        expandoCell: 1,
			        loadingMessage: messages.loading,
			        autoHeight: true,
			      }, document.createElement('div'));
			    this.domNode.appendChild(this._grid.domNode);
			    
			    this._grid.startup();
			    
			    // hide the user name column initially
			    this._grid.layout.setColumnVisibility(2, false);
			    
			    this.displayNoChangesetsIfNecessary();
			}
		},
		
		displayNoChangesetsIfNecessary: function()
		{
			if (this._grid == null)
				return;
			if (this._grid.store._arrayOfAllItems.length == 0)
			{
				this._grid.showMessage(messages.noChangesetsDefined);
			}
			else
			{
				this._grid.showMessage();
			}
		},
		
		escapeHtml: function(str){
			return jazz.util.html.escape(str);
		},
		
		formatUserName: function(value, inRowIndex, level, cell)
		{
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			var iconURL = this.getImageUrl(item, model, cell.grid);
			
			var label = "";
			if (value) {
				label = this.escapeHtml(value);
			}
			return "<div>" + label + "</div>";
		},
		
		formatName: function(value, inRowIndex, level, cell)
		{
			var model =  cell.grid.treeModel;
			var item = cell.grid.getItem(inRowIndex);
			var iconURL = this.getImageUrl(item, model, cell.grid);
			
			var label = "";
			if (value) {
				label = this.escapeHtml(value);
			}
			// disable opening the changeset in an editor
//			return "<div ondblclick=\"window.location.href=\'#action=jazz.viewPage&id=WorkspaceEditor\';\" ><img src="+"\""+iconURL+"\"" + " style=\"padding-top: 2px; padding-right: 4px;\"  height=\"16px\" width=\"16px\"\>" + label+"</div>";
			return "<div><img src="+"\""+iconURL+"\"" + " style=\"padding-top: 2px; padding-right: 4px;\"  height=\"16px\" width=\"16px\"\>" + label+"</div>";
		},
		
		getImageUrl: function(item, model, grid){
			if (item.type == "changeset")
			{
				if (item.state == States.Active){
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset.gif').uri;
				} else if (item.state == States.Completed) {
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_complete.gif').uri;
				} else if (item.state == States.Committed) {
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_committed.gif').uri;
				} else {
					return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_suspended.gif').uri;
				}
			}
			else if (item.type == "workitem")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/workitem.gif').uri;
			else if (item.type == "folder")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/open_folder.gif').uri;
			else if (item.type == "resource")
				return dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/resource.gif').uri;
			return "";
		},
		
		_performAction: function(node, action)
		{
			var resourceUri = node.uri;
			var encodedUri = encodeURI(resourceUri);
			var headers;
			var name;
			var jsonPostData = { "action" : action }
			if (action == Actions.Commit){
				headers = {
						"Content-Type" : "text/plain",
						"VvcChangeSetCommitTarget" : ContextUtils.getContextUri()
				};
			} else if (action == Actions.Rename) {
				name = prompt(messages.renamePrompt, node.title);
				if (!name || name == node.title) {
					// if user has cancelled the input prompt
					return;
				}
				
				headers = {
						"Content-Type" : "text/plain"
				};
				jsonPostData["VvcChangesetRenameNewName"] = name;
			} else {
				headers = {
						"Content-Type" : "text/plain"
				};				
			}

			jazz.client.xhrPost({
				handleAs: 'text',
				headers: headers,
				sync: true,
				url: resourceUri, //$NON-NLS-1$
				postData: JSON.stringify(jsonPostData),

				load: dojo.hitch(this, function(name, action, response, ioArgs) {
					this._grid.store.fetchItemByIdentity({
						identity:node.uri,
						onItem:dojo.hitch(this._grid.store, function(name, action, item){
							if (action == Actions.Rename){
								this.setValue(item, 'title', name);
							} else if (action == Actions.Commit) {
								this.setValue(item, 'state', "COMMITTED");
							} else if (action == Actions.Complete){
								this.setValue(item, 'state', "COMPLETED");
							}
							this.save();
						}, name, action)
					});						
				}, name, action),
				
				error: dojo.hitch(this, function(e) {
					console.log(e);
					if (this.displayErrorMessage)
						this.displayErrorMessage("Error: "+e.message);
					else
						alert('error '+e.message);
				})
			});
		},

		_discard: function(node)
		{
			var resourceUri = node.uri;
			jazz.client.xhrDelete({
				handleAs: 'text',
				sync: true,
				url: resourceUri, //$NON-NLS-1$
				content: {
					
				},

				load: dojo.hitch(this, function(response, ioArgs) {
					this._grid.store.fetchItemByIdentity({
						identity:node.uri,
						onItem:dojo.hitch(this._grid.store, function(item){
							this.deleteItem(item);
							this.save();
						})
					});
				}),
				
				error: dojo.hitch(this, function(e) {
					console.log(e);
					if (this.displayErrorMessage)
						this.displayErrorMessage("Error: "+e.message);
					else
						alert('error '+e.message);
				})
			});
		},

		
		_actionRegistry: { 
			changeset : [
			    // Not implemented yet         
/*				{
					isApplicable: function(node) {
						return true; 
					},
					label:  messages.addWorkItem,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/add_workitem.gif').uri,
					handler: function (node) {
						alert('TODO: add workitem');
					}
				}, 
				{
					isApplicable: function(node) {
						return node.state != States.Suspended;
					},
					label:  messages.suspend,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/suspend.gif').uri,
					handler: function (node) {
						alert('TODO: suspend');
					}
				},
				{
					isApplicable: function(node) {
						return node.state == States.Suspended;
					},
					label: messages.activate,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/activate.gif').uri,
					handler: function (node) {
						alert('TODO: activate');
					}
				},*/
			    {
					isApplicable: function(node) {
						return (node.state != States.Committed);
					},
					label:  messages.discard,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/delete.gif').uri,
					handler: function (node) {
						if (node.state == States.Completed) {
							var yes = confirm(messages.discardCompletedChangeset);
							if (!yes) {
								return;
							}
						}
						this._discard(node);
					}
				},              
				{
					isApplicable: function(node) {
						return node.state != States.Committed && node.resources && node.resources.length > 0;
					},
					label:  messages.share,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/share_change.gif').uri,
					handler: function (node) {
						this._performAction(node, Actions.Commit);
					}
				},
				{
					isApplicable: function(node) {
						return (node.state != States.Completed && node.state != States.Committed && node.resources && node.resources.length > 0);
					},
					label: messages.complete,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/complete.gif').uri,
					handler: function (node) {
						this._performAction(node, Actions.Complete);
					}
				},
				{
					isApplicable: function(node) {
						return node.state != States.Committed;
					},
					label: messages.rename,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/rename.gif').uri,
					handler: function (node) {
						this._performAction(node, Actions.Rename);
					}
				}
				
			],
			resource: [
/* Hide ui without implementation before release
				{
					isApplicable: function(node) {
						return true; 
					},
					label:  messages.move,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/move.gif').uri,
					handler: function (node) {
						alert('TODO: move');
					}
				},              
				{
					isApplicable: function(node) {
						return true;
					},
					label:  messages.open,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/open.gif').uri,
					handler: function (node) {
						alert('TODO: open resource');
					}
				}
*/
			],
			workitem: [
				{
					isApplicable: function(node) {
						return true; 
					},
					label:  messages.unlink,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/unlink.gif').uri,
					handler: function (node) {
						alert('TODO: disassociate work item');
					}
				},              
				{
					isApplicable: function(node) {
						return true;
					},
					label:  messages.open,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/open.gif').uri,
					handler: function (node) {
						alert('TODO: open work item');
					}
				}
			],
		},

		
		formatActions: function(value, inRowIndex, level, cell) 
		{
			var model =  cell.grid.treeModel;
			var toolbar = new jazz.ui.Toolbar();
			var node = cell.grid.getItem(inRowIndex);
			var self = this;
			
			var actionRegistry = this._actionRegistry[node.type];
			
			if (actionRegistry)
			{
				dojo.forEach(actionRegistry, dojo.hitch(this, function (action)
				{
					var button = jazz.ui.toolbar.Button({
						title: action.label,
						imgSrc: action.imgSrc,
						onClick: dojo.hitch(this, function(){
							dojo.hitch(self,action.handler)(node);
						})
					});
					if (!action.isApplicable(node))
						button.set("enabled", false);
					toolbar.addChild(button);
				}));
			}
			dojo.addClass(toolbar.domNode, "com-ibm-team-vvc-changesets-explorerActionsToolbar");		
			return toolbar;
		},
		
		resize: function()
		{
			this._grid.resize();
		},
		
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
		},
		
		collapseAll:function()
		{
			for (var i = this._grid.cache.items.length -1; i >= 0; i--) {
				if (this._grid.cache.getExpandoStatusByRowIndex(i)){
					this.toggleExpandRow(i);
				}
			}
		},
		
		getView: function() {
			return this._grid.views.views[1];
		},
		
		toggleExpandRow: function(ind) {
			var idty = this._grid._by_idx[ind].idty; 
			var expando = this.getView()._expandos[idty];
			if (expando) {
				expando.onToggle({stopPropagation: function() {}, preventDefault: function() {}} , true);
			}
		},
		
		setUser: function(user) {
			if (user != this._user) {
				this._user = user;
				if (this._isShowAllUser()) {
					this._grid.layout.setColumnVisibility(2, true);
				} else {
					this._grid.layout.setColumnVisibility(2, false);
				}
				this.refresh();
			}
		},
		
		_isShowAllUser: function() {
			if (this._user === "*") {
				return true;
			}
			return false;
		},
		
		setShowCommitted: function(show) {
			this._showCommitted = show; 
		},
		
		_getData: function()
		{
			this.data = null;
			
			if (this.hideErrorMessage)
				this.hideErrorMessage();
			
			var contextUri = ContextUtils.getContextUri();
			var content = {
					withResources : "true",
					includeCreated : "true",
					includeDisplayNames: "true",
					configuration : contextUri
			};
			
			if (! this._showCommitted) {
				content.filterCommitted = "true";
			}

			if (this._user && this._user.length > 0) {
				content.user = this._user;
			}
			
			jazz.client.xhrGet({
				handleAs: 'json',
				sync: true,
				url: contextRoot + '/vvcshared/proxy', //$NON-NLS-1$
				headers : {
					vvc_servicePath : "/changesets/query"
				},
				content: content,

				load: dojo.hitch(this, function(result) {
					this.data = result;
				}),
				
				error: dojo.hitch(this, function(e) {
					console.log(e);
					if (this.displayErrorMessage)
						this.displayErrorMessage("Error: "+e.message);
					else
						alert('error '+e.message);
				})
			});
			
			return this.data; 
		},
		
		_computeResourceId: function(changesetUri, resourceUri){
			//To get an unique identifier we need to take the 2 last segments
			var resourceUriWithoutLastSegment = resourceUri.substring(0, resourceUri.lastIndexOf("/"));
			var resourceIdx = resourceUriWithoutLastSegment.lastIndexOf("/")+1;
			var resourceId = resourceUri.substring(resourceIdx).replace("/", "_");
			return changesetUri + "#" + resourceId;
		},
		
		_formatData: function(data)
		{
			var processedData = [];
			// data may be null the first time coming to vvc due to the authentication process.
			if (data){
				var i, j;
				for (i=0; i<data.length; i++) {
					data[i].type= 'changeset';
					data[i].id = data[i].uri;
					var resources = data[i].resources;
					if (resources){
						for ( j=0; j < resources.length; j++) {
							var title = resources[j].displayName;
							if (!title || title.length < 1) {
								title =  resources[j].concept;
							}
							resources[j].title= title;
							resources[j].type='resource';
							resources[j].id = this._computeResourceId(data[i].id, resources[j].concept);					
						}
					}
					processedData.push(data[i]);
				};
			}
			return { identifier: 'id',
					label: 'title',
					items: processedData					
			};
			
		},
		
		_getModel: function()
		{
/*			var data = { identifier: 'title',
			        label: 'title',
			        items: [
			          { title:'Untitled Change Set', type:'changeset', state: 'active', resources:
			        	  [{ title:'resources1', label: 'Resources', type:'folder', resources:
				        	  [
				        	  		{ title:'com.ibm.team.vvc.shared.web/src', type:'resource'},
				        	  		{ title:'com.ibm.team.vvc.shared.web/src/foo.c', type:'resource'},
				        	  		{ title:'com.ibm.team.vvc.shared.web/src/boo.c', type:'resource'},
				        	  ]
				          }]
			          },
			          { title:'25135: Change set view - Create new changeset widget', type:'changeset', state: 'suspend', resources:
			        	  [{ title: 'resources2', label:'Resources', type:'folder', resources:
				        	  [
				        	  		{ title:'com.ibm.team.vvc.shared.web/plugin.xml', type:'resource'},
				        	  		{ title:'com.ibm.team.vvc.shared.web/resources/ui/changesets', type:'resource'},
				        	  		{ title:'com.ibm.team.vvc.shared.web/resources/ui/changesets/nls/ChangesetsMessages.js', type:'resource'},
				        	  ]
				          },
				          { title: '25135: Change set view', type:'workitem'
				          },
				          { title: '25133: Finish Beta 1', type:'workitem'
				          }]
			          },
			          ]
			    };*/
			
			var data = this._formatData(this._getData());
			var store = new dojo.data.ItemFileWriteStore({data: data});
		    return new dojox.grid.LazyTreeGridStoreModel({store: store, deferItemLoadingUntilExpand:true, childrenAttrs: ['resources']});
					    
		},
		
		refresh: function()
		{
			this._showMessage("Refreshing changesets");
			var model = this._getModel();
			this._grid.setModel(model);
			this._hideMessage();
		},
		
		_showMessage: function(message)
	    {
	    	dojo.publish(jazz.ui.PageTemplate.StatusMessage.SHOW, [message]);
	    	var pageTemplate = dijit.byId("jazz_ui_PageTemplate_0");
	    	if (pageTemplate)
	    	{ //eliminate fading in animation because sometimes loading is too quick to show the message box
	    		pageTemplate._fadeInAnim.gotoPercent(1.0);
	    	}
	    	
	    },
		    
	    _hideMessage: function()
	    {
	    	dojo.publish(jazz.ui.PageTemplate.StatusMessage.HIDE);
	    },
	    
	    addChangeset: function(newChangeset)
	    {
	    	if (this._grid != null)
	    	{
	    		this._grid.store.newItem(arguments[0]);
				this._grid.store.save();
	    	}
	    },
	});   
	})();
})();