/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");
dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dijit.tree.ForestStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");
dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require('jazz.util.html');

dojo.require('dojo.i18n');
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.ui.Toolbar");
dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid");
dojo.require("com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm");
dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");

dojo.require("jazz.util.ResizeNotifier");
dojo.require("jazz.app.ua.PageHoverHelp");

dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");

var singleUserIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/user.gif').uri;
var allUsersIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/all_users.gif').uri;
var changesetIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset.gif').uri;
var committedIcon = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changeset_committed.gif').uri;

(function ()
{
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var ContextUtils = 	com.ibm.team.vvc.shared.web.ui.context.ContextUtils;
	
	dojo.declare("com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsWidgit", [dijit._Widget, dijit._Templated], {
	  
		widgetsInTemplate: true,
		
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'ui/changesets/templates/ChangesetsWidgit.html'),
	   
		constructor: function() {
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.changesets", "ChangesetsMessages");
			
			this.explorerUrl = dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/changesets.gif').uri;
			
			this._updateTitle();
		},
		
		postCreate: function(){
			var self = this;
			
			this._constructToolbar();
			this._constructGrid();
			this._initializeCSH();
			this._initializeChangesetSubscribers();
			
			this._resizeHandle = jazz.util.ResizeNotifier.register({
				context: this,
				callback: function() {
		    		setTimeout(function(){
		    			self.resizeExplorer();
		    	    }, 100);
		    	},
				force: true});
		},
		
		_updateTitle: function() {
			this._configurationContext = this._getConfigurationContext();
			var title;
			if (this._configurationContext) {
				title = this._configurationContext.getTitle();
			}
			if (title) {
				this._initialized = true;
			} else {
				title = "";
			}
			this.changesetsHeading = dojo.string.substitute(this.messages.changesetsHeading, [ jazz.util.html.escape(title) ]);
		},
		
		_initializeChangesetSubscribers: function()
		{
			dojo.subscribe("vvc.changeset_created", dojo.hitch(this, function() 
			{
				if (this._grid != null)
				{
					this._grid.addChangeset(arguments[0]);
				}
			} ));
		},
		
		_initializeCSH: function() {	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.shared.web.page_ChangesetsExplorer001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		_getConfigurationContext: function() {
			var contextUri = ContextUtils.getContextUri();
			var configurationContext = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			configurationContext.load(contextUri);
			return configurationContext;
		},
		
		_constructToolbar:function()
		{
			if (!this._toolbar)
			{
				var toolbar = new jazz.ui.Toolbar();
				
				var button = jazz.ui.toolbar.Button({
					title: this.messages.switchUserModeTooltip,
					imgSrc: singleUserIcon,
					onClick: dojo.hitch(this, function(){
						if (!this.showAll || this.showAll === false) {
							this.showAll = true;
							this._grid.setUser("*");
							this.showChangesetsButton._img.src = allUsersIcon;
						} else {
							this.showAll = false;
							this._grid.setUser(null);
							this.showChangesetsButton._img.src = singleUserIcon;
						}
					})
				});
				toolbar.addChild(button);
				this.showChangesetsButton = button;
				
				this.showChangesetsButtonSeparator  = new jazz.ui.toolbar.Separator();
				toolbar.addChild(this.showChangesetsButtonSeparator);
				
				// hide the button initially
				this.showChangesetsButton.domNode.style.display = "none";
				this.showChangesetsButtonSeparator.domNode.style.display = "none";

				this.committedButton = jazz.ui.toolbar.Button({
					title: this.messages.showCommittedTooltip,
					imgSrc: committedIcon,
					onClick: dojo.hitch(this, function(){
						if (!this.showCommitted || this.showCommitted === false) {
							this.showCommitted = true;
							this._grid.setShowCommitted(true);
							this.committedButton._img.src = changesetIcon;
						} else {
							this.showCommitted = false;
							this._grid.setShowCommitted(false);
							this.committedButton._img.src = committedIcon;
						}
						this._grid.refresh();
					})
				});
				toolbar.addChild(this.committedButton);
				
				this.showCommittedButtonSeparator  = new jazz.ui.toolbar.Separator();
				toolbar.addChild(this.showCommittedButtonSeparator);
							
				var newChangesetButton = jazz.ui.toolbar.Button({
					title: this.messages.newChangeset,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/create_change.gif').uri,
					onClick: dojo.hitch(this, function(){
						var form = new com.ibm.team.vvc.shared.web.ui.changesets.CreateChangesetForm({"configuration" : ContextUtils.getContextUri()});
						form.show();
					})
				});
				toolbar.addChild(newChangesetButton);
				
				toolbar.addChild(new jazz.ui.toolbar.Separator());
				
				var refreshButton = jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.shared.web', 'graphics/refresh.gif').uri,
					onClick: dojo.hitch(this, function(){
						this._grid.refresh();
					})
				});
				toolbar.addChild(refreshButton);
				
				this._toolbar = toolbar;
				this._toolbarDiv.appendChild(toolbar.domNode);
			}
		},
		
		setIsAdminUser: function(isAdminUser)
		{
			this.isAdminUser = isAdminUser;
			if (isAdminUser != true) {
				// hide the show all user tool button
				if (this.showChangesetsButton) {
					this.showChangesetsButton.domNode.style.display = "none";
				}
				if (this.showChangesetsButtonSeparator) {
					this.showChangesetsButtonSeparator.domNode.style.display = "none";
				}
			} else {
				if (this.showChangesetsButton) {
					this.showChangesetsButton.domNode.style.display = "";
				}
				if (this.showChangesetsButtonSeparator) {
					this.showChangesetsButtonSeparator.domNode.style.display = "";
				}
			}
		},
		
		_constructGrid: function() {
			if (!this._grid)
			{
				this._grid = new com.ibm.team.vvc.shared.web.ui.changesets.ChangesetsTreeGrid({
					"displayErrorMessage" : this.displayErrorMessage,
					"hideErrorMessage" : this.hideErrorMessage
				});
				
			    /* append the new grid to the div */
			    this._explorerDiv.appendChild(this._grid.domNode);
		
			    /* Call startup() to render the grid */
			    this.resizeExplorer();
			}
		},
		
		resizeExplorer: function()
	    {
	    	if (this._grid)
		 		this._grid.resize();
	    },
	    
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
	    	if (this._breadcrumbs) {
				this._breadcrumbs.destroyRecursive();
				delete this._breadcrumbs;
			}
	    	if (this._toolbar) {
				this._toolbar.destroyRecursive();
				delete this._toolbar;
			}
		},
		
		// The changeset view may has problem getting the configuration
		// the first time the user go from the application to vvc due to 
		// the authentication process. So, we need to update the title and
		// content when this happen.
		initialize: function() {
			if (this._initialized === true) {
				return;
			}
			this._initialized = true;
			
			// update the title
			var childNodes = this._pageHeader.childNodes;
			for (var i in childNodes) {
				var node = childNodes[i];
				if (node.nodeName === "#text") {
					this._configurationContext = this._getConfigurationContext();
					var title;
					if (this._configurationContext) {
						title = this._configurationContext.getTitle();
						if (title) {
							this.title = title;
							this.changesetsHeading = dojo.string.substitute(this.messages.changesetsHeading, [ jazz.util.html.escape(title) ]);
							node.nodeValue = this.changesetsHeading;
						}
					}
					break;
				}
			}

			// refresh the content
			if (this._grid) {
				this._grid.refresh();
			}
		},
		
		displayErrorMessage: function(errorMessage) 
		{
			var mb = new jazz.ui.MessageBox();
			var node = dojo.byId("_changesetPageErrorMessages");
			if (node) {
				mb.placeAt(node);
				mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
				dojo.style("_changesetPageErrorMessages", {display: ""});
			}
		},
		
		hideErrorMessage: function() {
			if (dojo.byId("_changesetPageErrorMessages")) {
				dojo.empty("_changesetPageErrorMessages");
				dojo.style("_changesetPageErrorMessages", {display: "none"});
			}
		},
	});   
	})();
})();