/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {	

	dojo.provide("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.Setting");
	
	dojo.require("jazz.client.xhr");
	dojo.require("net.jazz.ajax.contextRoot");
	dojo.requireLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
		 
	var Setting = com.ibm.team.vvc.shared.core.ui.objects.Setting;
	
	var _spaceMapSetting = null;
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.shared.web.ui.context", "ConfigurationContextMessages");
	
	com.ibm.team.vvc.shared.web.ui.context.ContextUtils = {
		
		MAX_RECENTLY_USED_SIZE : 5,
		spaceUri : null,
		_rootVvcUri: null,
		
		setSpace: function(space) {
			this.spaceUri = space;
		},
		
		getSpace: function() {
			return this.spaceUri;
		},
		
		getUserSetting: function() {
			if (!_spaceMapSetting) {
				// First build the setting URI
				var settingURI = null;
				// '/queryvvc/vvcurl' has to exist on the hosted application.
				var args = {
					handleAs: 'json',
					url: net.jazz.ajax.contextRoot() + '/queryvvc/vvcurl', //$NON-NLS-1$
					sync: true,
					load: function(response, ioArgs) {
						return response;
					}
				};
				var def = jazz.client.xhrGet(args);
				var location = def.ioArgs.xhr.getResponseHeader("Location");
				if (def.ioArgs.xhr.status == 200 && location) {
					settingURI = location + "/settings/spaceMap";
				}
				// Then get the setting
				if (settingURI) {
					_spaceMapSetting = new Setting({
						uri: settingURI
					});
					_spaceMapSetting.load(settingURI);
				}
			}
			return _spaceMapSetting;
		},
		
		getCurrentContext: function()
		{
			if (!this.spaceUri) {
				return null;
			}
			
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}

			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.currentContextUri) {
				var config = this.getConfiguration(entry.currentContextUri);
				if (config != null)
				{
					return config;
				}
				else
				{
					delete entry.currentContextUri;
					if (setting) {
						setting.setData(spaceMap);
						setting.save();						
					}
				}
			}
			return null;
		},
							
		getConfiguration: function(configurationUri)
		{
			var loaded = true;
			var config = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var uri = this.getVvcRootUri() + "/repository/configuration?uri="+configurationUri;
			config.load(uri, function(response, ioArgs) { loaded = false; console.info("Could not load '"+configurationUri+"' Response:"+response.message); } );
			if (loaded)
				return config;
			return null;
		},
		
		setCurrentContext: function(configuration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}
			
			if (configuration && configuration.uri) {
				if (entry.currentContextUri === configuration.uri) {
					return; // current context is not changed
				}
				entry.currentContextUri = configuration.uri;
			}
			
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
			
			dojo.publish("vvc.context_updated", [configuration]);
		},
		
		getRecentlyUsed: function()
		{
			return this._loadRecentlyUsedUris(this._getRecentlyUsedUris());
		},
		
		_getRecentlyUsedUris: function()
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (entry && entry.recentlyUsedUris) {
				return entry.recentlyUsedUris;
			} else {
				return [];
			}
		},
		
		_loadRecentlyUsedUris: function(recentlyUsedUris)
		{
			var recentlyUsedConfigurations = [];
			for (i in recentlyUsedUris)
			{
				var recentlyUsedUri = recentlyUsedUris[i];
				if (recentlyUsedUri != null)
				{
					var config = this.getConfiguration(recentlyUsedUri);
					if (config != null)
						recentlyUsedConfigurations.push(config);
					else
						this._removeConfigurationFromRecentlyUsed(recentlyUsedUri);
				}
			}
			return recentlyUsedConfigurations;
		},
		
		_removeConfigurationFromRecentlyUsed: function(configUri)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			var i = entry.recentlyUsedUris.length - 1;
			while (i >= 0)
			{
				if (entry.recentlyUsedUris[i] == configUri)
					entry.recentlyUsedUris.splice(i, 1);
				i--;
			}
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		addConfigurationToRecentlyUsed: function(configuration)
		{
			var spaceMap = null;
			var setting = this.getUserSetting();
			if (setting) {
				spaceMap = setting.getData();
			}
			if (!spaceMap) {
				spaceMap = [];
			}
			
			var entry = this.findSpaceEntry(spaceMap);
			if (!entry) {
				entry = {};
				entry.spaceUri = this.spaceUri;
				spaceMap.push(entry);
			}			
			
			if (!entry.recentlyUsedUris) {
				entry.recentlyUsedUris = [];
			}
			
			if (this.isConfigurationInRecentlyUsed(configuration))
				return;
			
			entry.recentlyUsedUris.unshift(configuration.uri);
			while (entry.recentlyUsedUris.length > this.MAX_RECENTLY_USED_SIZE)
			{
				entry.recentlyUsedUris.pop();
			}
			
			if (setting) {
				setting.setData(spaceMap);
				setting.save();				
			}
		},
		
		isConfigurationInRecentlyUsed: function(configuration)
		{
			var recentlyUsedUris = this._getRecentlyUsedUris();
			if (recentlyUsedUris) {
				for (i in recentlyUsedUris)
				{
					if (configuration.uri == recentlyUsedUris[i])
						return true;
				}
			}
			return false;
		},
		
		getConfigurationIconClass: function(configuration) {
			if (configuration.isSnapshot())
				return "com-ibm-team-vvc-context-snapshot";
			else if (configuration.isChangeset())
			{
				if (configuration.isMutable())
					return "com-ibm-team-vvc-context-changeset";
				else
					return "com-ibm-team-vvc-context-completed-changeset";
			}
			return "com-ibm-team-vvc-context-workspace";
		},
		
		findSpaceEntry: function(spaceMap) {
			if (this.spaceUri != null && spaceMap != null) {
				for (var i in spaceMap) {
					var entry = spaceMap[i];
					if (entry && entry.spaceUri) {
						if (entry.spaceUri == this.spaceUri) {
							return entry;
						}
					}
				}
			}
			return null;
		},
		
		// the 2 functions below are copied from PageUtils.js
		getContextUri: function() {
			var uri = this.getRequestParmValue("contextUri");
			if (uri != null) {
				uri = unescape(uri);
			}
			return uri;			
		},
		
		getRequestParmValue: function(paramName, uri) {
			var params = uri;
			if (!params) {
				params = window.location.href;
			}
			var i = params.indexOf("?");
			if (i !== -1) {
				params = params.substring(i+1);
				params = params.replace("#", "&");
			} else {
				i = params.indexOf("#");
				params = params.substring(i);
			}
			var nvps = [];
			var rawNvps = params.split("&");
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				if (nvp[0] === paramName) {
					return nvp[1];
				}
			}
			return null;
		},

		getVvcRootUri: function()
		{
			if (this._rootVvcUri == null)
			{
				this._cacheVvcRootUri();
				if (this._rootVvcUri == null)
					alert(messages.failedToRetrieveRootVvcUri);
			}
			return this._rootVvcUri;
		},
		_cacheVvcRootUri: function()
		{
			// '/queryvvc/vvcurl' has to exist on the hosted application.
			var args = {
				handleAs: 'json',
				url: net.jazz.ajax.contextRoot() + '/queryvvc/vvcurl', //$NON-NLS-1$
				sync: true,
				load: function(response, ioArgs) {
					return response;
				}
			};
			var deferred =  jazz.client.xhrGet(args);
			var location = deferred.ioArgs.xhr.getResponseHeader("Location");
			if (deferred.ioArgs.xhr.status == 200 && location) {
				this._rootVvcUri = location;
				if (this._rootVvcUri && this._rootVvcUri.charAt(this._rootVvcUri.length-1)=="/") {
					this._rootVvcUri = this._rootVvcUri.substring(0, this._rootVvcUri.length-2);
				}
			} else {
				console.log('error', response);
				this.displayErrorMessage("Invalid Response -- VVC Location not specified. (Status:"+deferred.ioArgs.xhr.status+")");
			}
		},		
	};


}());