/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page.SnapshotEditor");

	dojo.require("dijit._Widget");
	dojo.require("dijit._Templated");
	dojo.require("dijit.InlineEditBox");
	dojo.require("jazz.util.html");
	
	dojo.require("jazz.app.ua.PageHoverHelp");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer");
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource");
	dojo.require("com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace");
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager");
		
	dojo.require("net.jazz.ajax.ui.PlatformUI");
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.context.ContextUtils");
	var ContextUtils = com.ibm.team.vvc.shared.web.ui.context.ContextUtils;

	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget");
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "WorkspaceEditorMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.page", "WorkspaceEditorMessages");
	
	var CHANGESET_VIEW = "changesetview";
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page.SnapshotEditor", [com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget, dijit._Templated], {

		iconUrl: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/add.gif'),
		
		widgetsInTemplate: true,
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/internal/page/templates/SnapshotEditor.html'),

		_editableTitle: null,
		_snapshotUri: null,
		_snapshot: null,
		_parentUri: null,
		_parentConfig: null,
		_isCreateOperation: false,
		_editingSnapshot: null,
		_createCompleted: false,
		_style: null,

		constructor: function() {
			this.messages = messages;
			this._style = PageUtil.getRequestParmValue("style");
			if (!this._style) {
				var context = ContextUtils.getContextUri();
				if (context && context.indexOf("/changesets/") != -1) {
					this._style = CHANGESET_VIEW;
				}
			}
			if (this.isChangeSetView()) {
				this.snapshotIconUrl = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/changeset.gif').uri;
			} else {
				this.snapshotIconUrl = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/snapshot.gif').uri;
			}
		},

		postCreate: function() {
			// Register the page action
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction(PageUtil.SNAPSHOT_PAGE, this, "showPage");
			ActionRegistry.registerAction("vvc.createSnapshot", this, "showCreateSnapshot");
			var onresizeHandle = dojo.connect(window, "onresize", this, this.onResize);
			this._initializeTabs();
		},
		
		_initializeTabs: function()
		{
			this._tabManager = new com.ibm.team.vvc.shared.web.ui.widgets.tab.TabManager();
			var class_selectedTab= "Selected";
			this._tabManager.registerTab(this._tabOverview, this.overview_content, class_selectedTab);
			this._tabManager.registerTab(this._tabDimensions, this.dimensions_content, class_selectedTab);
			this._tabManager.registerTab(this._tabDependencies, this.dependencies_content, class_selectedTab, dojo.hitch(this, function() { this._dependenciesViewer.load(this);}));
			this._tabManager.registerTab(this._tabResources, this.resources_content, class_selectedTab, dojo.hitch(this, function() { this._resourceMappingTable.load(this);}));
		},
		
		_initializeCSH: function() {
			if (this._ua != null)
				this._pageHeader.removeChild(this._ua.domNode);
	   		if (this._isCreateOperation)
				cshKey =  "com.ibm.team.vvc.server.core.page_SnapshotEditorCreate001";
			else
				{
					if (this.isChangeSetView())
						cshKey =  "com.ibm.team.vvc.server.core.page_ChangesetEditorEdit001";
					else	
						cshKey =  "com.ibm.team.vvc.server.core.page_SnapshotEditorEdit001";
				}	
			this._ua = new jazz.app.ua.PageHoverHelp({uaKey: cshKey});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		isChangeSetView: function() {
			if (this._snapshot && this._snapshot.getBase() != null) {
				return true;
			}
			if (CHANGESET_VIEW === this._style) {
				return true;
			}
			return false;
		},

		showCreateSnapshot: function()
		{
			this._isCreateOperation = true;
			this._renderPage();
		},
		
		showPage: function() {
			this._isCreateOperation = PageUtil.isCreate();
			this._renderPage();
		},
		
		_hideNonCreationTabs: function()
		{
			this._tabDependencies.style.display = "none";
			this._tabResources.style.display = "none";
		},
		
		_renderPage: function() {
			this.hideMessage();
			this._initializeCSH();
			this._initBreadcrumbs();
			this.addButtons();
			this._initSnapshot();
			this.enableActions();
			
			if (this._isCreateOperation) {
				this._hideNonCreationTabs();
				
				this._initEditableTitleNode(this._titleNode, this.messages.newSnapshotName, this.messages.clickToEdit);
				
				// initialize the parent snapshot field
				var context = ContextUtils.getContextUri();
				if (context) {
					this._parentUri = unescape(context);
					this._parentConfig = this._getConfiguration(this._parentUri);
					parentName = this._parentConfig.getTitle();
				}
				else {
					parentName = "";
				}
				
				this._clearParentContext();
				
				if (parentName) {
					this.setParentWorkspaceField(parentName, context);
				}
				
				// build content of dimension tab
				var dimensions = [];
				var dimension = {};
				dimension.value = messages.purpose + "\n" + parentName;
				dimensions.push(dimension);
				this._buildDimensionTab(dimensions);
			}
			
			this._connectEventHandlers();
			
			if (this.isChangeSetView()) {
				var node = dojo.byId("dependencies");
				while (node) {
					if (node.tagName == "TD") {
						node.parentNode.removeChild(node);
						break;
					}
					node = node.parentNode;
				}
				return;
			}
			
			this._updateSpaceField();
		},
		
		_buildDimensionTab: function(dimensions) {
			// create table
			var columnData = [{ label: this.messages.name }, { label: this.messages.value }];
			var container = this._dimensionContainerNode;
			if (container && container.firstChild) {
				container.removeChild(container.firstChild);
			}
			
			// create table header
			var table = new jazz.ui.StyledTable({
				trim: jazz.ui.StyledTable.TrimStyles.PRIMARY_HEADLESS,
				columns: columnData
			});
			table.domNode.style.width = "100%";
			container.appendChild(table.domNode);
			
			if (!dimensions || dimensions.length == 0) {
				return;
			}

			// Now add the rows
			var tr, td;
			for (var i = 0; i < dimensions.length; i++){
				tr = document.createElement("tr");
				var nameValuePair = dimensions[i].value;
				var index = nameValuePair.indexOf("\n");
				if (index == -1) {
					continue;
				}
				var name = nameValuePair.substring(0, index);
				var value = nameValuePair.substring(index + 1);
				var link = null;
				var pageId = null;
				
				if ("TimeOrder" === name) {
					name = messages.timeOrderDimension;
					pageId = PageUtil.SNAPSHOT_PAGE;
					if (this._isCreateOperation && this._parentConfig) {
						link = this._parentConfig.getUri();
					} else if (this._snapshot) {
						link = this._snapshot.getParentSnapshotUri();
					}
				} else if ("Purpose" === name) {
					name = messages.purposeDimension;
					pageId = PageUtil.WORKSPACE_PAGE;
					if (this._isCreateOperation && this._parentConfig) {
						link = this._parentConfig.getUri();
					} else if (this._snapshot) {
						link = this._snapshot.getParentWorkspaceUri();
					}
				}
				
				// name column
				td = document.createElement("td");
				td.innerHTML = name;
				tr.appendChild(td);
				
				// value column
				td = document.createElement("td");
				if (link != null && pageId != null) {
					PageUtil.setLinkField(td, value, link, pageId);
				} else {
					td.innerHTML = jazz.util.html.escape(value);
				}
				tr.appendChild(td);
				table.tbody.appendChild(tr);
			}
			
			return table;
		},
		
		_clearParentContext: function() {
			while (this._parentContext.hasChildNodes()) {
				this._parentContext.removeChild(this._parentContext.lastChild);
			}
		},
		
		_initSnapshot: function()
		{
			var context = ContextUtils.getContextUri();
			if (this._isCreateOperation) {
				this._parentUri = unescape(context);
			} else {
				this._snapshotUri = unescape(context);
				var snapshot = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
				var link = PageUtil.getVvcUri() + "/repository/configuration?uri=" + this._snapshotUri;
				var deferred = snapshot.load(link, dojo.hitch(this, this.onLoadError));
				deferred.addCallback(this, function (param) {
					this._snapshot = snapshot;
					this.refresh();
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
				});
			}
		},
		
		_updateSpaceField: function()
		{
			if (this._spaceField) {
				this._configurationSpace.removeChild(this._spaceField);
			}
			
			var spaceTitle;
			if (this._snapshot) {
				spaceTitle = this._snapshot.getSpaceName();
			} else if (this._parentConfig) {
				spaceTitle = this._parentConfig.getSpaceName();
			}
			if (!spaceTitle) {
				var spaceUri = PageUtil.getRequestParmValue("spaceUri");
				if (spaceUri) 
				{
					this._spaceUri = unescape(spaceUri);
					var space = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationSpace();
					space.load(this._spaceUri);
					spaceTitle = space.getTitle();
				}
			}

			if (!spaceTitle) {
				spaceTitle = this.messages.globalSpaceTitle;
			}
			this._spaceField = dojo.doc.createTextNode(spaceTitle);
			this._configurationSpace.appendChild(this._spaceField);
		},
		
		_connectEventHandlers: function() {
			this.connect(this._descriptionField,"onchange", function () {
				this.toggleSaveButton();
			});
		},
		
		_initEditableTitleNode: function(titleNode, title, tooltip){
			if (this._editableTitle) {
				this._editableTitle.setValue(title);
			}
			else
			{
				titleNode.innerHTML = jazz.util.html.escape(title);
				this._editableTitle = new dijit.InlineEditBox({
					title: tooltip, 
					autoSave: true, 
					width: "700px",
					noValueIndicator: this.getEditableTitleNodeDetails(), 
					editorParams: {maxLength: "60"}, 
					onFocus: dojo.hitch(this, this.hideMessage),
					onChange: dojo.hitch(this, this.onTitleChange)}, titleNode
				);
	
				dojo.addClass(this._editableTitle.domNode, "vvc-editor-editIndicator");
				this._editableTitle.startup();
			}
			this._editableTitle.originalValue = title;
		},
		
		setTitleValue: function(value){
			if(this._editableTitle){
				this._editableTitle.set('value', value);			
			}
		},

		onTitleChange: function(e){
			this.toggleSaveButton();
		},

		getEditableTitleNodeDetails: function(){
			if (this.isChangeSetView()) {
				return this.messages.noname_changeset;
			}
			return "?";
		},
		
		addButtons: function() {
			var toolbar = dojo.byId("snapshot_editor_toolbar");
			if (!toolbar || toolbar.firstChild) {
				return;
			}
			var button = dojo.create("button");
			button.disabled = true;
			var buttonText = this.messages.saveAndClose;
			if (this._isCreateOperation) {
				buttonText = this.messages.create;
			}
			button.style.minWidth = "80px";
			button.appendChild(dojo.doc.createTextNode(buttonText));
			dojo.connect(button, "onclick", this, "onSave");
			toolbar.appendChild(button);
			this._saveButton = button;
			
			if (!this._cancelButton) {
				this._cancelButton = dojo.create("button");
				this._cancelButton.style.minWidth = "80px";
				this._cancelButton.style.marginLeft = "5px";
				this._cancelButton.appendChild(dojo.doc.createTextNode(this.messages.cancel));
				dojo.connect(this._cancelButton, "onclick", this, "onCancel");
				toolbar.appendChild(this._cancelButton);
			}
		},
		
		lastSegment: function(uri) {
			var i = uri.lastIndexOf('/');
			if (i !== -1) {
				var s = uri.substring(i+1);
				return s;
			}
			return "";
		},
		
		refresh: function() {
			// initialize the parent snapshot field
			if (this.isChangeSetView()) {
				this._parentUri = this._snapshot.getBase();
				if (this._parentUri != null) {
					var base = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
					base.load(this._parentUri);
					this.setParentWorkspaceField(base.getTitle(), this._parentUri);
				}
				
				// remove dependencies tab for change set view
				var node = this._tabDependencies;
				if (node && node.parentNode) {
					node.parentNode.removeChild(node);
				}
				node = dojo.byId("snapshotIcon");
				if (node.tagName == "IMG") {
					node.src = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/changeset.gif').uri;
				}
				if (!this._parentUri) { // is change set in completed/committed state?
					// remove non applicable tabs and fields
					var node = this._tabDimensions;
					if (node && node.parentNode) {
						node.parentNode.removeChild(node);
					}
					var creator = PageUtil.getUser(this._snapshot.getCreator());
					if (creator && creator.name) {
						this._setFieldNameAndValue(this._parentContext, this.messages.creator, creator.name);
					}
					var created = this._snapshot.getCreated();
					if (created) {
						var dateToFormat =  dojo.date.locale.format(new Date(created), { formatLength: 'medium' }); 
						this._setFieldNameAndValue(this._configurationSpace, this.messages.created, dateToFormat);
					}
				}
			} else {
				this._parentUri = this._snapshot.getParentWorkspaceUri();
				this.setParentWorkspaceField(this._snapshot.getParentWorkspaceName(), this._parentUri);
			}
			
			// create and set title
			var title = this._snapshot.getTitle();
			this._initEditableTitleNode(this._titleNode, title, this.messages.clickToEdit);
			
			// set description text
			this._descriptionField.value = this._snapshot.getDescription();
			this._descriptionField.originalValue = this._descriptionField.value;
			
			// build dimension viewer
			this._buildDimensionTab(this._snapshot.getDimensions());
			
			// build dependency viewer
			var dependenciesContainer = dojo.byId("dependencies_viewer_div");
			if (this._dependenciesViewer) {
				// refresh this view
				var node = this._dependenciesViewer.domNode;
				dependenciesContainer.removeChild(node);
			}
			
			var params = { style: 3, lazyStart: true };
			if (this._snapshot) {
				params.workspaceUri = this._snapshot.uri;
			}
			this._dependenciesViewer = new com.ibm.team.vvc.server.core.ui.snapshots.WorkspacesAndSnapshotsViewer(params);
			dependenciesContainer.appendChild(this._dependenciesViewer.domNode);
			
			// build resource mappings viewer
			var container = dojo.byId("resources_viewer_div");
			if (this._resourceMappingTable) {
				// refresh this view
				container.removeChild(this._resourceMappingTable.domNode);
			}
			
			var args = { configurationUri: this._snapshot.uri, lazyStart: true };
			if (this.isChangeSetView()) {
				args.showActions = false;
			}
			this._resourceMappingTable = new com.ibm.team.vvc.server.core.ui.internal.page.ResourceMappingViewer(args);
		    container.appendChild(this._resourceMappingTable.domNode);
		    
		    this._tabManager.showActiveTab(this._tabOverview);
		},
		
		_setFieldNameAndValue: function(field, fieldName, value) {
			if (field) {
				PageUtil.setLinkField(field, value);
				var tr = field.parentNode.parentNode;
				var textNode = tr.children[0].firstChild;
				textNode.nodeValue = fieldName;
			}
		},
		
		setParentWorkspaceField: function(parentWorkspaceName, uri, pageId) {
			PageUtil.setLinkField(this._parentContext, parentWorkspaceName, uri, PageUtil.WORKSPACE_PAGE);
		},
		
		onSave: function() {
			this.hideMessage();
			var title = this._editableTitle.value;
			var description = this._descriptionField.value;
			if (!title || title.length === 0) {
				this.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: this.messages.emptyName
				});
				return;
			}
			this.disableActions();
			var deferred;
			if (this._isCreateOperation) {
				UIUtils.showPageMessage(messages.creatingNewSnapshot);
				if (this._parentConfig) {
					var spaceUri = this._parentConfig.getSpaceUri();
					var configUri = PageUtil.findConfigurationName(title, spaceUri, false);
					if (configUri != null) {
						var config = this._getConfiguration(configUri);
						if (config != null && !config.isMutable()) {
							this.enableActions();
							return this._showDuplicateNameError();
						}
					}
				}
				deferred = PageUtil.createSnapshot(title, description, this._parentUri, null);
				deferred.addCallback(this, function(params){
					var returnLink = PageUtil.getReturnLink(params);
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.OK,
						summary: messages.createSuccessful + returnLink
					});
					this.disableSaveButton();
					this._createCompleted = true;
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
					this.enableActions();
				});
			} else {
				UIUtils.showPageMessage(messages.saving);
				this._snapshot.setTitle(title);
				if (this._snapshot.isTitleChanged()) {
					var configUri = PageUtil.findConfigurationName(title, this._snapshot.getSpaceUri(), false);
					if (configUri != null) {
						config = this._getConfiguration(configUri);
						if (config != null && !config.isMutable() && config.uri !== this._snapshot.uri) {
							this.enableActions();
							return this._showDuplicateNameError();
						}
					}
				}
				this._snapshot.setDescription(description);
				
				deferred = this._snapshot.save();
				deferred.addCallback(this, function(params){
					var returnLink = PageUtil.getReturnLink(this._snapshot.uri);
					this.showMessage({
						type: jazz.ui.MessageBox.STATUS_TYPES.OK,
						summary: messages.saveSuccessful+returnLink
					});
					this._editableTitle.originalValue = this._editableTitle.value;
					this._descriptionField.originalValue = this._descriptionField.value;
					this.enableActions();
				});
				deferred.addErrback(this, function(error) {
					this.showMessage(PageUtil.createXhrErrorMessage(deferred.ioArgs));
					this.enableActions();
				});
			}
		},
		
		onLoadError: function(response, ioArgs) {
			this.showMessage(PageUtil.createXhrErrorMessage(ioArgs));
		},
		
		enableSaveButton: function()
		{
			if (this._saveButton) {
				this._saveButton.disabled = false;
				var node = dojo.byId("dirty");
				if (!node.firstChild) {
					node.appendChild(dojo.doc.createTextNode(" *"));
				}
			}
		},
		
		disableSaveButton: function()
		{
			if (this._saveButton) {
				this._saveButton.disabled = true;
				var node = dojo.byId("dirty");
				if (node.firstChild) 
					node.removeChild(node.firstChild);
			}
		},
		
		toggleSaveButton: function()
		{
			if (this.isDirty())
				this.enableSaveButton();
			else
				this.disableSaveButton();
		},
		
		enableActions: function()
		{
			this.toggleSaveButton();
			if (this._cancelButton) {
				this._cancelButton.disabled = false;
			}
			if (this._editableTitle)
				this._editableTitle.setDisabled(false);
			if (this._descriptionField)	
				this._descriptionField.disabled = false;
		},
		
		disableActions: function()
		{
			this.disableSaveButton();
			if (this._cancelButton) {
				this._cancelButton.disabled = true;
			}
			this._editableTitle.setDisabled(true);
			this._descriptionField.disabled = true;
		},
		
		isDirty: function()
		{
			if (this._editableTitle && (this._editableTitle.value != this._editableTitle.originalValue))
				return true;
			else if (this._descriptionField && (this._descriptionField.value != this._descriptionField.originalValue))
				return true;
			return false;
		}
	})   
})();
