/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget");

	dojo.require("jazz.ui.Breadcrumbs");
	
	dojo.require("dijit._Widget");
	
	dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.require("com.ibm.team.vvc.shared.web.ui.util.UIUtils");
	var UIUtils = com.ibm.team.vvc.shared.web.ui.util.UIUtils;
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.internal.page._EditorWidget", [dijit._Widget], {

		messageBox: null,
		_messageBoxContainer: null,
		_resizableTables: [],
		
		getYPosition: function(element) {
			var yPosition = 0;
			if (element.offsetParent) {
				do {
					yPosition += element.offsetTop;
					if(!element.offsetParent)
						break;
					element = element.offsetParent;
				} while (true);
			} else if (element.y) {
				yPosition += element.y;
			}
			return yPosition;
		},
		
		addResizableChildElement: function(resizableElement) {
			if (resizableElement) {
				this._resizableTables.push(resizableElement);
				this._resizeDivNode(resizableElement);
			}
		},
		
		_resizeDivNode: function(divNode) {
			var yPosition = this.getYPosition(divNode);
			var paneHeight = document.body.offsetHeight - yPosition - 50 + "px";
			divNode.style.height = paneHeight;
		},
		
		onResize: function() {
			for (var i = 0; i < this._resizableTables.length; i++) {
				var node = this._resizableTables[i];
				if (node) {
					this._resizeDivNode(node);
				}
			}
		},
		
		showMessage: function(args) {
			UIUtils.hidePageMessage();
			if (args && this._messageBoxContainer) {
				args["closable"] = true;
				if (!this.messageBox) {
					this.messageBox = new jazz.ui.MessageBox();
					this._messageBoxContainer.appendChild(this.messageBox.domNode);
				}
				this.messageBox.showMessage(args);
				this._messageBoxContainer.style.display = "";
			}
		},
		
		hideMessage: function() {
			if (this._messageBoxContainer) {
				this._messageBoxContainer.style.display = "none";
			}
		},
		
		_getConfiguration: function(uri) {
			var configuration = new com.ibm.team.vvc.shared.core.ui.objects.ConfigurationResource();
			var link = PageUtil.getVvcUri() + "/repository/configuration?uri=" + uri;
			var deferred = configuration.load(link);
			return configuration;
		},
		
		_showDuplicateNameError: function() {
			this.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary: this.messages.duplicateConfigurationName
			});
		},
		
		onCancel: function() {
			history.back();
		},
		
		_initBreadcrumbs:function()
		{
			if (!this._breadcrumbs)
			{
				var parts = [];
				var segment = PageUtil.getClientAppBreadcrumbSegment();
				if (segment != null) {
					parts.push(segment);
				}
				
				var explorerPageUri = PageUtil.createLink(PageUtil.SHOW_EXPLORER);
				segment = { label: this.messages.configurations, href : explorerPageUri };
				parts.push(segment);
				
				// if breadcrumb is specified, append it. One of the case is
				// the create snapshot action from workspace page.
				var breadcrumb = PageUtil.getRequestParmValue("breadcrumb");
				if (breadcrumb) {
					breadcrumb = decodeURIComponent(unescape(breadcrumb));
					segment = {
						label: breadcrumb, 
						href : 'javascript:history.back()'
					};
					parts.push(segment);
				}
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
				
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
	})   
})();
