/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function() {

	dojo.provide("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");
	dojo.require("jazz.ui.Dialog");
	dojo.require("jazz.ui.MessageBox");
	dojo.require("dojox.xml.DomParser");
	
	dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "PageUtilMessages");
	var messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.internal.util", "PageUtilMessages");

	com.ibm.team.vvc.server.core.ui.internal.util.PageUtil = {

		DEFAULT_PAGE : "defaultPage",
		EDIT_CONFIGURATION_PAGE : "editConfiguration",
		EDIT_DIMENSION_PAGE : "editDimension",
		EDIT_DIMENSION_TYPE_PAGE : "editDimensionType",
		COMPARE_CONFIGURATIONS_PAGE : "CompareConfigurations",
		COMPARE_VIEWER_PAGE : "CompareViewer",
		WORKSPACE_PAGE : "WorkspaceEditor",
		SNAPSHOT_PAGE : "SnapshotEditor",
		ADD_DEPENDENCY_PAGE : "DependencyPicker",
		REBASE_PAGE : "RebasePicker",
		SHOW_EXPLORER: "vvc.showExplorer",

		CREATE_OPERATION: "operation=create",
		OPEN_OPERATION: "operation=open",
		
		ACTIVE_TAB_INDEX_COOKIE_NAME: "ActiveTabIndexListCookie",
		CLIENT_INTERFACE: "clientInterface",
	
		VVC_URI: null,
		
		isCreate: function() {
			var loc = window.location.href;
			if (loc.indexOf(this.CREATE_OPERATION) !== -1) {
				return true;
			}
			return false;
		},
		
		getVvcUri: function() {
			return net.jazz.ajax.BootstrapProperties.FRONTSIDE_URL;
		},
		
		navigateToMainPage: function() {
			// go back to VVC home page if we don't have a redirect URL.
			window.location.href = net.jazz.ajax._contextRoot;
		},
		
		getCurrentPageId: function() {
			return net.jazz.ajax.ui.PlatformUI.getWorkbench().currentPageId;
		},

		setWorkbenchTitle: function(title) {
			return net.jazz.ajax.ui.PlatformUI.getWorkbench().setTitle(title);
		},
		
		showMessage: function(msg) {
			 PlatformUI.getWorkbench().showMessage(msg);
		},
		
		openMessageDialog: function(title, messages) {
			if (this.dialog) this.dialog.destroy(); 
			
			var contentNode = dojo.create("span");
			this.dialog = new jazz.ui.Dialog({
		    	contentNode: contentNode,
		    	primaryTitle: title,
		    	width: "400px",
		    	closeOnBlur: true
			});
			contentNode.innerHTML = messages;
		},
		
		getUser: function(userUri) {
			var link = this.getVvcUri() + "/repository/user?uri=" + userUri;
			return this.httpGet(link);
		},
		
		httpGet: function(uri) {
			var result;
			var args = {
				url: uri,
				handleAs: "json",
				sync: true,
				headers: {
					Accept: "application/json;application/rdf+json;text/rdf+json;text/json"
				},
			};
			var deferred = jazz.client.xhrGet(args);
			deferred.addCallback(this, function (param) {
				result = param;
			});
			return result;
		},
		
		getClientAppBreadcrumbSegment: function() {
			var link = this.getRequestParmValue(this.CLIENT_INTERFACE);
			if (!link) {
				return;
			}
			link = unescape(link);
			var context = this.getRequestParmValue("context", link);
			if (!context) {
				return;
			}
			context = unescape(context);
			var name = this.getRequestParmValue("contextClientName", link);
			if (name) {
				name = decodeURIComponent(unescape(name));
			} else {
				name = messages.clientApplication
			}
			return {label: name, href: context };
		},
		
		getBaseline: function(){
			return jazz.app.context.get().baseline;
		},
		
		htmlEscape: function(text) {
			return jazz.util.html.escape(text);
		},
		
		getVvcWebUri: function() {
			var uri = this.getVvcUri();
			if (uri !== null) {
				return uri + net.jazz.ajax._webuiPrefix;
			}
			return null;
		},
		
		createWorkspace: function(name, description, parent, space) {
			return this._createConfigurationResource(name, description, parent, space, true);
		},
		
		createSnapshot: function(name, description, parent, space) {
			return this._createConfigurationResource(name, description, parent, space, false);
		},
		
		_createConfigurationResource: function(name, description, parent, space, mutable) {
			var data = {name: name, description: description, mutable: mutable, space: space};
			var childUri = "/configurations/child";
			if (parent != null) {
				childUri += "?oneGreaterThan=" + parent;
			}
			var args = {
					url: net.jazz.ajax.contextRoot() + childUri,
					headers: {
						"Accept" : "application/json",
						"Content-Type" : "application/json"
					},
					sync: true,
					handleAs: "json",
					postData: dojo.toJson(data),
					load: function(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
						return response;
					}
			};
			return jazz.client.xhrPost(args);
		},
		
		findConfigurationName: function(configName, spaceUri, mutable) {
			var serviceName;
			if (mutable) {
				serviceName = "findWorkspaceName";
			} else {
				serviceName = "findSnapshotName";
			}
			var uri = this.getVvcUri() + "/repository/" + serviceName + "?configurationName="
				+ configName + "&spaceUri=" + spaceUri + "&mutable=" + mutable;
			var args = {
				url: uri,
				headers: {
					"Accept" : "application/json"
				},
				sync: true,
				handleAs: "text",
				load: function(response, ioArgs) {
					if (ioArgs) {
						var xhr = ioArgs.xhr;
						if (xhr) {
							return xhr.getResponseHeader("Location");
						}
					}
				},
			};
			var deferred = jazz.client.xhrGet(args);
			var configUri = null;
			deferred.addCallback(this, function(param){
				if (param)
					configUri = param;
			});
			deferred.addErrback(function(error) {
				alert('error '+e.message);
			});
			return configUri;
		},
		
		getRequestParmValue: function(paramName, uri) {
			var params = uri;
			if (!params) {
				params = window.location.href;
			}
			var i = params.indexOf("?");
			if (i !== -1) {
				params = params.substring(i+1);
				params = params.replace("#", "&");
			} else {
				i = params.indexOf("#");
				params = params.substring(i);
			}
			var nvps = [];
			var rawNvps = params.split("&");
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				if (nvp[0] === paramName) {
					return nvp[1];
				}
			}
			return null;
		},
		
		getRequestParms: function(url) {
			if (!url) {
				url = new dojo._Url(window.location.href);
			} else if (typeof url === 'string') {
				url = new dojo._Url(url);
			}
			var nvps = [];
			var rawNvps = url.query ? url.query.split("&") : [];
			for(var i=0; i<rawNvps.length; i++) {
				var nvp = rawNvps[i].split("=");
				nvps.push({n: nvp[0], v: nvp[1]});
			}
			return nvps;
		},
		
		createXhrErrorMessage: function(ioArgs) {
			var xhrError = this.createXhrError(ioArgs);
			var msg = { type: jazz.ui.MessageBox.STATUS_TYPES.ERROR, summary: xhrError.message };
			if (xhrError.detailedMessage) {
				msg.details = xhrError.detailedMessage;
			}
			return msg;
		},
		
		createXhrError: function(ioArgs) {
			var contentType = ioArgs.xhr.getResponseHeader("Content-Type");
			if (ioArgs.xhr.responseText && contentType && contentType.toLowerCase() === "application/rdf+xml") {
				var rdf = dojox.xml.DomParser.parse(ioArgs.xhr.responseText);
				rdfDescription = rdf.getElementsByTagName("rdf:Description")[0];
				return this._createErrorObject(rdfDescription);
			}
			
			var xhrError = {};
			if (ioArgs.xhr.statusText) {
				xhrError.message = ioArgs.xhr.statusText + " - " + ioArgs.url;
			} else {
				xhrError.message = dojo.string.substitute(messages.defaultXhrErrorMessage, [ioArgs.url]);
			}
			
			if (ioArgs.xhr.responseText) {
				xhrError.detailedMessage = [ {message: ioArgs.xhr.responseText} ];
			}
			xhrError.date = ioArgs.xhr.getResponseHeader("Date");
			xhrError.status = ioArgs.xhr.status;
			return xhrError;
		},
		
		/* Error Object structure
			errorStatus      The major status code, usually the corresponding HTTP status code.
			errorMessage     Some textual response.
			errorSubStatus   The minor status code, usually a Foundation specified value.
			errorSeverity    One of the values: info, warning, error, critical.
			detailedMessage  Any additional information the application needs to convey.
			seeAlso          A URL to a resource that may contain information such as how to resolve this error.
			errorCause       Another err:Report that denotes a condition that caused this error report to be generated.
		 */
		_createErrorObject: function (rdfDescription) {
			var xhrError = {};		
			for (var i = 0; i < rdfDescription.childNodes.length; ++i) {
				var childNode = rdfDescription.childNodes[i];
				if (childNode.nodeType === 1 && childNode.childNodes.length > 0 && childNode.childNodes[0].nodeValue) {
					switch (String(childNode.nodeName)) {
						case 'err:errorStatus':
							xhrError.status = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorMessage':
							xhrError.message = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorSubStatus':
							xhrError.subStatus = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorSeverity':
							xhrError.severity = childNode.childNodes[0].nodeValue;
							break;
						case 'err:detailedMessage':
							var value = childNode.childNodes[0].nodeValue;
							xhrError.detailedMessage = [ {message: value} ];
							break;
						case 'err:seeAlso':
							xhrError.seeAlso = childNode.childNodes[0].nodeValue;
							break;
						case 'err:errorCause':
							xhrError.cause = this._createErrorObject(childNode.getElementsByTagName("rdf:Description")[0]);
							break;
					}
				}
			}
			
			return xhrError;
		},
		
		setLinkField: function(field, value, uri, pageId, tooltip) {
			if (field) {
				if (field.firstChild) {
					field.removeChild(field.firstChild);
				}

				if (uri) {
					uri = this.createLink(pageId, "operation=open&contextUri=" + uri);
					var link = dojo.create("a");
					link.href = uri;
					link.innerHTML = this.htmlEscape(value);
					if (tooltip) {
						link.title = tooltip;
					}
					field.appendChild(link);
				} else {
					var node = dojo.doc.createTextNode(value);
					field.appendChild(node);
				}
			}
		},

		createLink : function(pageId, queryString, fragmentString) {
			var loc = window.location.href;
			var i = loc.indexOf("?");
			var uri = loc;
			if (i !== -1) {
				uri = loc.substring(0, i);
			} else {
				i = loc.indexOf("#");
				if (i !== -1) {
					uri = loc.substring(0, i);
				}
			}
			
			// check for debug flag
			var debugFlag = "debug=true";
			if (loc.indexOf(debugFlag) !== -1) {
				queryString = this.addQueryParam(queryString, "debug", "true");
			}
			
			// keep client interface parameter
			var value = this.getRequestParmValue(this.CLIENT_INTERFACE);
			if (value) {
				queryString = this.addQueryParam(queryString, this.CLIENT_INTERFACE, value);
			}
			
			if (queryString) {
				uri += "?" + queryString;
			}
			if (pageId) {
				uri = uri + "#action=" + pageId;
				if (fragmentString) {
					uri += "&" + fragmentString;
				}
				return uri;
			}
		},
		
		addQueryParam: function(queryString, paramName, paramValue) {
			if (!queryString) {
				return paramName + "=" + paramValue;
			}
			return queryString + "&" + paramName + "=" + paramValue;
		},

		showPage : function(pageId, queryString, fragmentString) {
			window.location.href = this.createLink(pageId, queryString, fragmentString);
		},
		
		showAction : function(action, queryString) {
			this.showPage(action, queryString);
		},
		
		openEditor: function(type, uri, spaceUri)
		{
			if (type == "snapshot")
				page = this.SNAPSHOT_PAGE;
			else
				page = this.WORKSPACE_PAGE;
			this.showAction(page, "operation=open&contextUri=" + escape(uri)+"&spaceUri="+escape(spaceUri));
		},
		
		getReturnLink: function(configurationUri)
		{
			var returnUrl = this._getReturnUrl(configurationUri);
			
			if (returnUrl)
				return " <a href=\'"+returnUrl+"\'> "+this._getReturnTitle()+"</a>";
			return " <a href=\"#\" onclick=\"history.back()\" > "+messages.returnBack+"</a>";
		},
		
		_getReturnTitle: function()
		{
			var returnTitle = this.getRequestParmValue("redirectTitle");
			if (returnTitle)
				return returnTitle;
			return messages.returnBackToApplication;
		},
		
		_getReturnUrl: function(configurationUri) {
			var redirectUri = this.getRequestParmValue("redirectUri");
			if (redirectUri) {
				redirectUri = unescape(redirectUri);
				// insert the newly created configuration URI
				var i = redirectUri.indexOf('?');
				if (i == -1) {
					redirectUri += "?configurationUri=" + escape(configurationUri);
				} else {
					redirectUri = redirectUri.substring(0,i+1) + "configurationUri=" 
					+ escape(configurationUri) + "&"
					+ redirectUri.substring(i+1);
				}
				
				return redirectUri;
			}
			return null;
		},
	};

})();
