/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.vvc.shared.core.ui.objects.VvcObject");

	dojo.declare("com.ibm.team.vvc.shared.core.ui.objects.VvcObject", null, {

		uri: null,
		properties: null,
		data: null,
		loaded: false,

		constructor: function(args) {
			if (args) {
				dojo.safeMixin(this, args);
			}
		},
		
		getUri: function() {
			return this.uri;
		},

		isLoaded: function()
		{
			return this.loaded;
		},
		
		getArrayProperty: function(name) {
			var value = null;
			if (this.properties) {
				value = this.properties[name];
			}
			return value;
		},
		
		getProperty: function(name) {
			var value = null;
			if (this.properties) {
				var property = this.properties[name];
				if (property && property[0]) {
					value = property[0].value;
				}
			}
			return value;
		},

		setProperty: function(name, value, type) {
			if (!this.properties) {
				this.properties = {};
				if (!this.data) {
					this.data = {};
				}
				this.data[this.uri] = this.properties;
			}
			var property = this.properties[name];
			if (property && property[0]) {
				property[0].value = value;
			} else {
				var item = {};
				if (!type) {
					type = "literal";
				}
				item.type = type;
				item.value = value;
				
				var propertyValue = [];
				propertyValue.push(item);
				
				if (!property) {
					property = {};
				}
				
				this.properties[name] = propertyValue;
			}
		},

		getType: function() {
			return this.getProperty("http://www.w3.org/1999/02/22-rdf-syntax-ns#type");
		},

		// for sub class to override and return the front service name
		// such as "configurations", "dimensions", "definitions".
		getFrontServiceId: function() {
			return "?";
		},

		load: function(uri, onError) {
			var args = {
					url: net.jazz.ajax.contextRoot() + '/vvcshared/proxy'+
						//TODO: Temporary work around to get around firebug ajax caching problem, because firebug is so dumb as to not respect the response headers
						'?right_now='+new Date().getTime(), //$NON-NLS-1$   
					headers: {
						"Accept" : "application/json;application/rdf+json;text/rdf+json;text/json",
						"vvc_servicePath" : uri
					},
					sync: true,
					handleAs: "json",
					load: dojo.hitch(this, this.onLoad),
					error: dojo.hitch(this, this.onError),
					failOk: true
			};
			
			if (onError != null)
				args.error = onError;
			return jazz.client.xhrGet(args);
		},

		onLoad: function(response, ioArgs) {
			if (response) {
				this.data = response;
				for (var uri in response) {
					this.uri = uri;
					this.properties = response[uri];
					break;
				}
				this.loaded = true;
			}
			return this;
		},

		onError: function(response, ioArgs) {
			alert("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'");
			console.log("Failed to load resource '"+ioArgs.args.headers.vvc_servicePath+"'...Invalid Response Status("+ioArgs.xhr.status+"): "+ response.message);
		},
		
		doSave: function() {
			var uri = this.getUri();
			var saveData = dojo.toJson(this.data);
			var args = {
					url: net.jazz.ajax.contextRoot() + '/vvcshared/proxy', //$NON-NLS-1$
					headers: {
						"Accept" : "application/rdf+json",
						"Content-Type" : "application/rdf+json",
						"vvc_servicePath" : uri
					},
					sync: true,
					handleAs: "json",
					putData: saveData,
					load: function(response, ioArgs) {
						if (ioArgs) {
							var xhr = ioArgs.xhr;
							if (xhr) {
								return xhr.getResponseHeader("Location");
							}
						}
					},
					error: dojo.hitch(this, this.onError),
			};
			return jazz.client.xhrPut(args);
		}

	});
})();