/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
dojo.provide("com.ibm.team.vvc.server.core.ui.workspaces.ConfigurationsLazyTreeGrid");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");

dojo.declare("com.ibm.team.vvc.server.core.ui.workspaces.ConfigurationsLazyTreeGrid", dojox.grid.LazyTreeGrid, {

	_buttonInx: 0,

	constructor: function() {
		this.toolbars = [];
	},
	
	destroy: function() {
		if (this._setFocusIndexHandle) {
			dojo.disconnect(this._setFocusIndexHandle);
		}
	},
	
	postCreate: function() {
		this.inherited(arguments);
		this._setFocusIndexHandle = dojo.connect(this.focus, "setFocusIndex", this, "_setFocusIndex" );
	},
	
	// accessibility -- if user tabs to first column, show toolbar
	_setFocusIndex: function(inRowIndex, inCellIndex) {
		if (inCellIndex == 0 || inCellIndex == 1) {
			var toolbar = this.toolbars[inRowIndex];			
			var buttons = toolbar.getChildren();
			if (buttons.length > 0) {
				// select row
				this.selection.deselectAll();
				this.selection.select(inRowIndex);
				
				// show toolbar
				if (inCellIndex === 0) {
					dojo.addClass(toolbar.domNode, "visibleToolbar");
					
					// remember toolbar for key actions
					this._toolbar = toolbar;
					this._buttons = buttons;
					this._enabledButtons = [];
					for (var i=0; i<this._buttons.length; i++) {
						if (this._buttons[i].enabled) {
							this._enabledButtons.push(i);
						}
					}
					if (this._buttonInx > this._enabledButtons.length-1) {
						this._buttonInx = this._enabledButtons.length-1;
					}
					this._refreshButtons();
				} else {
					// remember row if user presses enter
					this._openLinkOnRow = inRowIndex;
				}
			}
		}

	},
	
	// accessibility-- if toolbar is shown because of tabbing, process left/right/enter keys to highlight/click on buttons
	onKeyDown: function(e) {
		var processed = false;
		var dk = dojo.keys;
		if ( !(e.altKey || e.metaKey)) {
			if (this._buttons) {
				switch(e.keyCode){
					case dk.LEFT_ARROW:
						if (this._buttonInx>0) {
							processed = true;
							this._buttonInx--;
							this._refreshButtons();
						}
						break;
					case dk.RIGHT_ARROW:
						if (this._buttonInx < this._enabledButtons.length-1) {
							processed = true;
							this._buttonInx++;
							this._refreshButtons();
						}
						break;
					case dk.ENTER:
						this._buttons[this._enabledButtons[this._buttonInx]].onClick();
						processed = true;
						break;
				}
			}
			if (this._openLinkOnRow) {
				switch(e.keyCode){
					case dk.ENTER:
						if (e.shiftKey) {
							var item = this.getItem(this._openLinkOnRow);
							com.ibm.team.vvc.server.core.ui.internal.util.PageUtil.openEditor(item.type, item.uri, item.spaceUri);
						} else {
							processed = true;							
						}
						break;
					default:
						processed = e.shiftKey;
						break;
				}
			}
		}
		if (processed) {
			dojo.stopEvent(e);
		} else {
			if (this._buttons) {
				for (var i=0; i<this._enabledButtons.length; i++) {
					this._buttons[this._enabledButtons[i]]._setEnabledAttr(true);
				}
				delete this._buttons;
				if (this._toolbar.domNode) {
					dojo.removeClass(this._toolbar.domNode, "visibleToolbar");
				}
			}
			delete this._openLinkOnRow;
			this._noResize = true;
			this.inherited(arguments);
			this._noResize = false;
		}
	},
	
	sizeChange: function(){
		// no grid resize on key down event-- messes up accessibility selected cell (DOJO bug)
		if (!this._noResize) {
			this.inherited(arguments);
		}
	},
	
	// accessibility-- use enabled state to indicate which button is active
	_refreshButtons: function() {
		for (var i=0; i<this._enabledButtons.length; i++) {
			this._buttons[this._enabledButtons[i]]._setEnabledAttr(i===this._buttonInx);
		}
	}
	
});

}());