/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 * 
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

(function(){
   
dojo.provide("com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("dojox.grid.LazyTreeGrid");
dojo.require("dijit.tree.ForestStoreModel");
dojo.require("dojo.data.ItemFileWriteStore");

dojo.require('dojo.i18n');
dojo.require("jazz.ui.Breadcrumbs");
dojo.require("jazz.ui.toolbar.Separator");
dojo.require("jazz.ui.Toolbar");
dojo.require("jazz.app.ua.PageHoverHelp");

dojo.require("com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesTreeGrid");

dojo.require("com.ibm.team.vvc.server.core.ui.internal.util.PageUtil");

dojo.require("jazz.util.ResizeNotifier");

dojo.requireLocalization("com.ibm.team.vvc.server.core.ui.workspaces", "WorkspacesPageMessages");

(function ()
{
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	
	var PageUtil = com.ibm.team.vvc.server.core.ui.internal.util.PageUtil;
	
	dojo.declare("com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesPage", [dijit._Widget, dijit._Templated], {
	  
		widgetsInTemplate: true,
		
		templatePath: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'ui/workspaces/templates/WorkspacesPage.html'),
	   
		constructor: function() {
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.vvc.server.core.ui.workspaces", "WorkspacesPageMessages");
			
			this.explorerUrl = dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/configurations.gif');
		},
		
		postCreate: function(){
			var self = this;
			// Register the page action
			var ActionRegistry = PlatformUI.getWorkbench().getActionRegistry();
			ActionRegistry.registerAction("vvc.showExplorer", this, "showConfigurations");
			ActionRegistry.registerAction("vvc.refreshExplorer", this, "refreshConfigurations");
			
			this._initializeCSH();
			
			this._resizeHandle = jazz.util.ResizeNotifier.register({
				context: this,
				callback: function() {
		    		setTimeout(function(){
		    			self.resizeExplorer();
		    	    }, 100);
		    	},
				force: true});
		},
	   	
	   	_initializeCSH: function() {
	   		this._ua = new jazz.app.ua.PageHoverHelp({uaKey: "com.ibm.team.vvc.server.core.page_ConfigurationsExplorer001"});
			this._pageHeader.appendChild(this._ua.domNode);
			this._ua.domNode.style.margin = "0px 0px 0px 5px";	
	   	},
	   	
		showConfigurations: function() {
			this._constructBreadcrumbs();
			this._constructToolbar();
			this._constructGrid();
		},
		
		refreshConfigurations: function() {
			this.showConfigurations();
			if (this._grid)
			{
				this._grid.refresh();
			}
		},
		
		_constructBreadcrumbs:function()
		{
			if (!this._breadcrumbs)
			{
				var parts = [];
				var segment = PageUtil.getClientAppBreadcrumbSegment();
				if (segment != null) {
					parts.push(segment);
				}
				
				var uri = PageUtil.createLink(PageUtil.SHOW_EXPLORER);
				segment = { label: this.messages.configurations, href : uri };
				parts.push(segment);
				
				this._breadcrumbs = new jazz.ui.Breadcrumbs({segments: parts });
				
				this._breadcrumbs.placeAt(dojo.byId("breadcrumbs"));
			}
		},
		
		_constructToolbar:function()
		{
			if (!this._toolbar)
			{
				var toolbar = new jazz.ui.Toolbar();
				
				var newWorkspaceButton = jazz.ui.toolbar.Button({
					title: this.messages.newWorkspace,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/create_workspace-enabled.gif').uri,
					onClick: dojo.hitch(this, function(){
						PageUtil.showAction("vvc.createWorkspace");
					})
				});
				toolbar.addChild(newWorkspaceButton);
				
				var self = this;
				var compareButton = jazz.ui.toolbar.Button({
					title: this.messages.compareWithEachOther,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/compare-enabled.gif').uri,
					onClick: dojo.hitch(this, function() {
						self._grid._compareSelected();
					})
				});
				toolbar.addChild(compareButton);
				
				toolbar.addChild(new jazz.ui.toolbar.Separator());
				
				/*var previousButton = jazz.ui.toolbar.Button({
					title: this.messages.previous,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/previous.gif').uri,
					onClick: dojo.hitch(this, function(){
						alert('TODO: previous');
					})
				});
				toolbar.addChild(previousButton);
				
				var nextButton = jazz.ui.toolbar.Button({
					title: this.messages.next,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/next.gif').uri,
					onClick: dojo.hitch(this, function(){
						alert('TODO: next');
					})
				});
				toolbar.addChild(nextButton);*/
				
				var collapseButton = jazz.ui.toolbar.Button({
					title: this.messages.collapse,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/collapse_all.gif').uri,
					onClick: dojo.hitch(this, function(){
						this._grid.collapseAll();
					})
				});
				toolbar.addChild(collapseButton);
				
				toolbar.addChild(new jazz.ui.toolbar.Separator());
				
				var refreshButton = jazz.ui.toolbar.Button({
					title: this.messages.refresh,
					imgSrc: dojo.moduleUrl('com.ibm.team.vvc.server.core', 'graphics/refresh.gif').uri,
					onClick: dojo.hitch(this, function(){
						this._grid.refresh();
					})
				});
				toolbar.addChild(refreshButton);
				
				this._toolbar = toolbar;
				this._toolbarDiv.appendChild(toolbar.domNode);
			}
		},
		
		_constructGrid: function() {
			if (!this._grid)
			{
				this._grid = new com.ibm.team.vvc.server.core.ui.workspaces.WorkspacesTreeGrid({
					"displayErrorMessage" : this.displayErrorMessage,
					"hideErrorMessage" : this.hideErrorMessage
				});
				
			    /* append the new grid to the div */
			    this._explorerDiv.appendChild(this._grid.domNode);
		
			    /* Call startup() to render the grid */
			    this.resizeExplorer();
			}
			else
			{
				this._grid.refresh();
			}
		},
		
		resizeExplorer: function()
	    {
		 	this._grid.resize();
	    },
	    
	    uninitialize: function()
		{
	    	if (this._grid) {
				this._grid.destroyRecursive();
				delete this._grid;
			}
	    	if (this._breadcrumbs) {
				this._breadcrumbs.destroyRecursive();
				delete this._breadcrumbs;
			}
	    	if (this._toolbar) {
				this._toolbar.destroyRecursive();
				delete this._toolbar;
			}
		},
		
		displayErrorMessage: function(errorMessage) 
		{
			var mb = new jazz.ui.MessageBox();
			mb.placeAt(dojo.byId("_workspacePageErrorMessages"));
			mb.showMessage({summary: errorMessage, type: jazz.ui.MessageBox.STATUS_TYPES.ERROR});
			dojo.style("_workspacePageErrorMessages", {display: ""});
		},
		
		hideErrorMessage: function() {
			dojo.empty("_workspacePageErrorMessages");
			dojo.style("_workspacePageErrorMessages", {display: "none"});
		},
	})   
	})();
})();